@@ -85,7 +85,7 @@ class ConfigManager:
         try:
             self.global_config = yaml.load(open(self.global_config_file_path, 'r', encoding='utf-8'), yaml.SafeLoader)
         except:
-            raise DSOException(CLI_MESSAGES['InvalidDSOConfigurationFile'])
+            raise DSOException(MESSAGES['InvalidDSOConfigurationFile'])
 
         self.update_merged_config()
 
@@ -97,7 +97,7 @@ class ConfigManager:
         if not os.path.exists(self.local_config_file_path):
             self.local_config = {}
             if show_warnings:
-                Logger.warn(CLI_MESSAGES['NoDSOConfigFound'])
+                Logger.warn(MESSAGES['NoDSOConfigFound'])
             return
 
         Logger.debug(f"Local DSO configuration found: path={self.local_config_file_path}")
@@ -105,7 +105,7 @@ class ConfigManager:
         try:
             self.local_config = yaml.load(open(self.local_config_file_path, 'r', encoding='utf-8'), yaml.SafeLoader)
         except:
-            raise DSOException(CLI_MESSAGES['InvalidDSOConfigurationFile'])
+            raise DSOException(MESSAGES['InvalidDSOConfigurationFile'])
 
         self.update_merged_config()
 
@@ -124,48 +124,74 @@ class ConfigManager:
             try:
                 config = yaml.load(open(configFile, 'r', encoding='utf-8'), yaml.SafeLoader)
             except:
-                raise DSOException(CLI_MESSAGES['InvalidDSOConfigurationFile'])
+                raise DSOException(MESSAGES['InvalidDSOConfigurationFile'])
             
             self.inherited_config = merge_dicts(config, self.inherited_config)
 
         self.update_merged_config()
 
     def override_config(self, config_overrides):
-
         self.overriden_config = {}
-
         if config_overrides:
-            try:
-                configs = config_overrides.split(',')
-                for config in configs:
-                    key = config.split('=')[0].strip()
-                    value = config.split('=')[1].strip()
-                    old = get_dict_item(self.merged_config, key.split('.'))
-                    if not old:
-                        Logger.warn(f"DSO configuration '{key}' was not found in the merged configuration.")
-                    else:
-                        Logger.warn(f"DSO configuration '{key}' was overriden from '{old}' to '{value or 'default'}'.")
-                    set_dict_value(self.overriden_config, key.split('.'), value)
-            except:
-                raise DSOException(CLI_MESSAGES['InvalidDSOConfigOverrides'])
+            configs = flatten_dict(config_overrides)
+            for key, value in configs.items():
+                old = get_dict_item(self.merged_config, key.split('.'))
+                if not old:
+                    Logger.warn(f"DSO configuration '{key}' was not found in the merged configuration.")
+                else:
+                    Logger.debug(f"DSO configuration '{key}' was overriden from '{old}' to '{value or 'default'}'.")
+                set_dict_value(self.overriden_config, key.split('.'), value)
 
         self.update_merged_config()
 
 
-    def load(self, working_dir, config_overrides):
+    def dict_to_config_string(self, dic):
+        flat = flatten_dict(dic)
+        print(flat)
+        return reduce(lambda x, y: f"{x[0]}={x[1]},{y[0]}={y[1]}", flat)
+
+
+    def config_string_to_dict(self, config_string):
+        if not config_string: return {}
+
+        result = {}
+        try:
+            configs = config_string.split(',')
+            for config in configs:
+                key = config.split('=')[0].strip()
+                value = config.split('=')[1].strip()
+                set_dict_value(result, key.split('.'), value)
+        except:
+            raise DSOException(MESSAGES['InvalidDSOConfigOverrides'])
+
+        return result
+
+    def get_config_overrides(self, context, config_overrides_string):
+        _config = {}
+        if context == 'global':
+            _config['project'] = 'default'
+            Logger.warn("Switched to the global context.")
+        elif context == 'project':
+            _config['application'] = 'default'
+            Logger.warn(f"Switched to the project context '{self.project}'.")
+        elif context == 'application':
+            pass
+        return merge_dicts(self.config_string_to_dict(config_overrides_string), _config)
+
+    def load(self, working_dir, context, config_overrides_string):
         self.working_dir = working_dir
         self.load_local_config()
         self.load_global_config()
         self.load_inherited_config()
-        self.override_config(config_overrides)
+        self.override_config(self.get_config_overrides(context, config_overrides_string))
         self.check_version()
 
     def check_version(self):
         if not int(self.merged_config['version']) == int(_default_config['version']):
             if int(self.merged_config['version']) > int(_default_config['version']):
-                Logger.warn(CLI_MESSAGES['DSOConfigNewer'].format(_default_config['version'], self.merged_config['version']))
+                Logger.warn(MESSAGES['DSOConfigNewer'].format(_default_config['version'], self.merged_config['version']))
             else:
-                Logger.warn(CLI_MESSAGES['DSOConfigOlder'].format(_default_config['version'], self.merged_config['version']))
+                Logger.warn(MESSAGES['DSOConfigOlder'].format(_default_config['version'], self.merged_config['version']))
 
 
     def get_provider_default_spec(self, provider_id):
@@ -217,7 +243,8 @@ class ConfigManager:
             result  = os.getenv('DSO_APPLICATION') or 'default'
         if self.project == 'default':
             if not result == 'default':
-                Logger.warn(f"The specific context of application '{result}' was ignored because the global context was used.")
+                # Logger.warn(f"Switched to the project context '{self.project}' from the application context '{result}'.")
+                Logger.debug(f"The application context '{result}' was reset because the global context is being used.")
                 result = 'default'
         return result.lower()
 
@@ -227,6 +254,8 @@ class ConfigManager:
             result = self.merged_config['project'] or 'default'
         else:
             result  = os.getenv('DSO_PROJECT') or 'default'
+        # if result == 'default':
+        #         Logger.warn(f"The global context is being used.")
         return result.lower()
 
     @property
@@ -234,7 +263,7 @@ class ConfigManager:
         try:
             result = self.merged_config['parameter']['provider']['id']
         except KeyError:
-            raise DSOException(CLI_MESSAGES['ProviderNotSet'].format('Parameter'))
+            raise DSOException(MESSAGES['ProviderNotSet'].format('Parameter'))
         return result
 
     @property
@@ -242,7 +271,7 @@ class ConfigManager:
         try:
             result = self.merged_config['secret']['provider']['id']
         except KeyError:
-            raise DSOException(CLI_MESSAGES['ProviderNotSet'].format('Secret'))
+            raise DSOException(MESSAGES['ProviderNotSet'].format('Secret'))
         return result
 
     @property
@@ -250,14 +279,14 @@ class ConfigManager:
         try:
             result = self.merged_config['template']['provider']['id']
         except KeyError:
-            raise DSOException(CLI_MESSAGES['ProviderNotSet'].format('Template'))
+            raise DSOException(MESSAGES['ProviderNotSet'].format('Template'))
         return result
 
     def parameter_spec(self, key=None):
         try:
             result = self.merged_config['parameter']['provider']['spec']
         except KeyError:
-            raise DSOException(CLI_MESSAGES['ProviderNotSet'].format('Parameter'))
+            raise DSOException(MESSAGES['ProviderNotSet'].format('Parameter'))
         if not key:
             return result
         elif key in result:
@@ -269,7 +298,7 @@ class ConfigManager:
         try:
             result = self.merged_config['secret']['provider']['spec']
         except KeyError:
-            raise DSOException(CLI_MESSAGES['ProviderNotSet'].format('Secret'))
+            raise DSOException(MESSAGES['ProviderNotSet'].format('Secret'))
         if not key:
             return result
         elif key in result:
@@ -281,7 +310,7 @@ class ConfigManager:
         try:
             result = self.merged_config['template']['provider']['spec']
         except KeyError:
-            raise DSOException(CLI_MESSAGES['ProviderNotSet'].format('Template'))
+            raise DSOException(MESSAGES['ProviderNotSet'].format('Template'))
         if not key:
             return result
         elif key in result:
@@ -299,9 +328,9 @@ class ConfigManager:
     def register_template_custom_render_path(self, key, render_path):
         result = get_dict_item(self.local_config, ['template', 'render']) or {}
         # if os.path.isabs(render_path):
-        #     raise DSOException(CLI_MESSAGES['AbsTemplateRenderPath'].format(render_path))
+        #     raise DSOException(MESSAGES['AbsTemplateRenderPath'].format(render_path))
         if os.path.isdir(render_path):
-            raise DSOException(CLI_MESSAGES['InvalidRenderPathExistingDir'].format(render_path))
+            raise DSOException(MESSAGES['InvalidRenderPathExistingDir'].format(render_path))
         result[key] = render_path
         self.local_config['template']['render'] = result
         self.flush_local_config()
