@@ -6,7 +6,7 @@ REGEX_PATTERNS = {
     # 'parameter_key_value' : r"^([a-zA-Z][a-zA-Z0-9_.-/]*)=(.*)$",
 }
 
-CLI_MESSAGES = {
+MESSAGES = {
     'InvalidKey': "'{0}' is an invalid key. Must conform to '{1}'",
     'ParameterNotFound': "Parameter '{0}' not found in the given context.",
     'SecretNotFound': "Secret '{0}' not found in the given context.",
