@@ -1,6 +1,6 @@
 from .logger import Logger
-from .config import Config
-from .providers import StoreProvider, ProviderManager
+from .config import Configs
+from .providers import StoreProvider, Providers
 from .stages import Stages
 
 class ParameterProvider(StoreProvider):
@@ -13,17 +13,17 @@ class ParameterProvider(StoreProvider):
     def get(self, stage, key):
         raise NotImplementedError()
 
-class ParametersClass():
+class ParameterManager():
 
     def validate_key(self, key):
-        provider = ProviderManager.ParameterProvider()
+        provider = Providers.ParameterProvider()
         Logger.info("Start validating parameter key...")
         return provider.validate_key(key)
 
     def list(self, stage, uninherited):
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.ParameterProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.ParameterProvider()
         if uninherited:
             Logger.info(f"Start listing uninherited SSM parameters: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
@@ -32,34 +32,34 @@ class ParametersClass():
 
     def add(self, stage, key, value):
         self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.ParameterProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.ParameterProvider()
         Logger.info(f"Start adding parameter: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}, value={value}")
         return provider.add(project, application, stage, key, value)
 
     def get(self, stage, key, revision):
         # self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.ParameterProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.ParameterProvider()
         Logger.info(f"Start getting parameter value: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
         return provider.get(project, application, stage, key, revision)
 
     def history(self, stage, key):
         # self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.ParameterProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.ParameterProvider()
         Logger.info(f"Start getting parameter history: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
         return provider.history(project, application, stage, key)
 
     def delete(self, stage, key):
         # self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.ParameterProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.ParameterProvider()
         Logger.info(f"Start deleting parameter: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
         return provider.delete(project, application, stage, key)
 
-Parameters = ParametersClass()
\ No newline at end of file
+Parameters = ParameterManager()
\ No newline at end of file
