@@ -37,19 +37,18 @@ class AwsSsmParameterProvider(ParameterProvider):
 ###--------------------------------------------------------------------------------------------
 
     def get_parameter_prefix(self, project, application, stage, key=None):
-        # output = f"/dso/{project}/{application}/{stage}"
-        output = "/dso"
-        output += f"/{project}"
+        result = "/dso/parameters"
+        result += f"/{project}"
         ### every application must belong to a project, no application overrides allowed in the default project
         if not project == 'default':
-            output += f"/{application}"
+            result += f"/{application}"
         else:
-            output += "/default"
+            result += "/default"
         stage = Stages.normalize(stage)
-        output += f"/{stage}"
+        result += f"/{stage}"
         if key:
-            output += f"/{key}"
-        return output
+            result += f"/{key}"
+        return result
 
 
 ###--------------------------------------------------------------------------------------------
@@ -73,9 +72,9 @@ class AwsSsmParameterProvider(ParameterProvider):
         ### check children parameters
         path = self.get_parameter_prefix(project, application, stage, key)
         # parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['String']},{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
-        parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
-        if len(parameters['Parameters']) > 0:
-            raise DSOException("Parameter key '{0}' is not allowed in the given stage becasue it would overwrite all the parameters in '{0}.*', such as '{0}.{1}'.".format(key,parameters['Parameters'][0]['Name'][len(path)+1:]))
+        response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
+        if len(response['Parameters']) > 0:
+            raise DSOException("Parameter key '{0}' is not allowed in the given context becasue it would overwrite '{0}.{1}' and all other parameters in '{0}.*' scope if any.".format(key,response['Parameters'][0]['Name'][len(path)+1:]))
 
         ### check parent parameters
         scopes = key.split('.')
@@ -84,21 +83,22 @@ class AwsSsmParameterProvider(ParameterProvider):
             path = self.get_parameter_prefix(project, application, stage, subKey)
             Logger.debug(f"Describing parameters: path={path}")
             # parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Type', 'Values':['String']},{'Key':'Name', 'Values':[path]}])
-            parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
-            if len(parameters['Parameters']) > 0:
-                raise DSOException("Parameter key '{0}' is not allowed in the given stage becasue it would overwrite parameter '{1}'.".format(key, subKey))
+            response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
+            if len(response['Parameters']) > 0:
+                raise DSOException("Parameter key '{0}' is not allowed in the given context becasue it would overwrite parameter '{1}'.".format(key, subKey))
 
 ###--------------------------------------------------------------------------------------------
 
     def locate_parameter(self, project, application, stage, key, uninherited=False):
         Logger.debug(f"Locating SSM parameter: project={project}, application={application}, stage={stage}, key={key}")
-        paths = self.get_hierachy_paths(project, application, stage, key, uninherited)
-        Logger.debug(f"SSM paths to search in order: {paths}")
+        paths = list(reversed(self.get_hierachy_paths(project, application, stage, key, uninherited)))
+        # Logger.debug(f"SSM paths to search in order: {paths}")
         for path in paths:
             Logger.debug(f"Describing SSM parameters: path={path}")
             # result = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['String']},{'Key':'Name', 'Values':[path]}])
-            result = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
-            if len(result['Parameters']) > 0: return result['Parameters']
+            response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
+            if len(response['Parameters']) > 0: return response['Parameters']
+        return []
 
 ###--------------------------------------------------------------------------------------------
 
@@ -111,8 +111,8 @@ class AwsSsmParameterProvider(ParameterProvider):
             if key in parameters:
                 Logger.warn("Inherited parameter '{0}' overridden.".format(key))
             parameters[key] = {'Value': parameter['Value'], 
-                                'Path': path,
-                                'Revision': parameter['Version'],
+                                'Path': parameter['Name'][:-len(key)-1],
+                                'Version': parameter['Version'],
                                 'Date': parameter['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                             }
         return parameters
@@ -124,40 +124,42 @@ class AwsSsmParameterProvider(ParameterProvider):
         if uninherited:
             paths.append(self.get_parameter_prefix(project, application, stage, key))
         else:
-            ### check /dso/project/application/stage/env
-            paths.append(self.get_parameter_prefix(project, application, stage, key))
-            if not Stages.is_stage_default_env(stage): ### otherwise already added above
-                ### check /dso/project/application/stage/default
-                paths.append(self.get_parameter_prefix(project, application, Stages.get_stage_default_env(stage), key))
-            if not Stages.is_default(stage): ### otherwise already added above
-                ### check /dso/project/application/default
-                 paths.append(self.get_parameter_prefix(project, application, Stages.default_stage(), key))
-            if not application == 'default': ### otherwise already added above
-                ### check /dso/project/default/stage/env
-                paths.append(self.get_parameter_prefix(project, 'default', stage, key))
-                if not Stages.is_stage_default_env(stage): ### otherwise already added above
-                    ### check /dso/project/default/stage/default
-                    paths.append(self.get_parameter_prefix(project, 'default', Stages.get_stage_default_env(stage), key))
-                if not Stages.is_default(stage): ### otherwise already added above
-                    ### check /dso/project/default/default
-                    paths.append(self.get_parameter_prefix(project, 'default', Stages.default_stage(), key))
-                if not project == 'default': ### otherwise already added above
-                    ### check /dso/default/default/stage/env
+            ### Add the global context: /dso/default/default/default/0
+            paths.append(self.get_parameter_prefix('default', 'default', 'default/0', key))
+            if not Stages.is_default(stage):
+                ### Add global stage context
+                paths.append(self.get_parameter_prefix('default', 'default', Stages.get_stage_default_env(stage), key))
+                ### Add global numbered stage context
+                if not Stages.is_stage_default_env(stage):
                     paths.append(self.get_parameter_prefix('default', 'default', stage, key))
-                    if not Stages.is_stage_default_env(stage): ### otherwise already added above
-                        ### check /dso/default/default/stage/default
-                        paths.append(self.get_parameter_prefix('default', 'default', Stages.get_stage_default_env(stage), key))
-                    if not Stages.is_default(stage): ### otherwise already added above
-                        ### check /dso/default/default/default
-                        paths.append(self.get_parameter_prefix('default', 'default', Stages.default_stage(), key))
+
+            if not project == 'default':
+                ### Add the project context: /dso/project/default/default/0
+                paths.append(self.get_parameter_prefix(project, 'default', 'default/0', key))
+                if not Stages.is_default(stage):
+                    ### Add the project stage context: /dso/project/default/stage/0
+                    paths.append(self.get_parameter_prefix(project, 'default', Stages.get_stage_default_env(stage), key))
+                    ### Add the project numbered stage context: /dso/project/default/stage/env
+                    if not Stages.is_stage_default_env(stage):
+                        paths.append(self.get_parameter_prefix(project, 'default', stage, key))
+                
+                if not application == 'default':
+                    ### Add the application context: /dso/project/application/default/0
+                    paths.append(self.get_parameter_prefix(project, application, 'default/0', key))
+                    if not Stages.is_default(stage):
+                        ### Add the project stage context: /dso/project/application/stage/0
+                        paths.append(self.get_parameter_prefix(project, application, Stages.get_stage_default_env(stage), key))
+                        ### Add the application numbered stage context: /dso/project/application/stage/env
+                        if not Stages.is_stage_default_env(stage):
+                            paths.append(self.get_parameter_prefix(project, application, stage, key))
 
         return paths
 
 ###--------------------------------------------------------------------------------------------
 
     def list(self, project, application, stage, uninherited):
-        ### construct search path in hierachy with no key specified in reverse order
-        paths = list(reversed(self.get_hierachy_paths(project, application, stage, None, uninherited)))
+        ### construct search path in hierachy with no key specified
+        paths = self.get_hierachy_paths(project, application, stage, None, uninherited)
         # Logger.debug(f"SSM paths to search in order: {paths}")
         parameters = {}
         for path in paths:
@@ -168,8 +170,9 @@ class AwsSsmParameterProvider(ParameterProvider):
             result['Parameters'].append({'Key': item[0], 
                                         'Value': item[1]['Value'], 
                                         'Path': item[1]['Path'],
-                                        'Revision': item[1]['Revision'],
+                                        'RevisionId': str(item[1]['Version']),
                                         'Date': item[1]['Date'],
+                                        # 'Version': item[1]['Version'],
                                         })
 
         return result
@@ -178,15 +181,17 @@ class AwsSsmParameterProvider(ParameterProvider):
 
     def add(self, project, application, stage, key, value):
         self.assert_no_scope_overwrites(project, application, stage, key)
-        found = self.locate_parameter(project, application, stage, key, True)
-        if found and len(found) > 0 and not found[0]['Type'] in ['String']:
-            raise DSOException(f"Failed to add parameter '{key}' becasue there is already a SSM secret with the same key in the given stage.")
+        found = self.locate_parameter(project, application, stage, key, uninherited=True)
+        if found and not found[0]['Type'] == 'String':
+            raise DSOException(f"Failed to add parameter '{key}' becasue becasue the key is not available in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         path = self.get_parameter_prefix(project, application, stage=stage, key=key)
         Logger.info(f"Adding SSM parameter: path={path}")
         response = ssm.put_parameter(Name=path, Value=value, Type='String', Overwrite=True)
         return {'Key': key, 
-                'Path': path,
-                'Revision': response['Version'],
+                'Value': value,
+                'Path': path[:-len(key)-1],
+                'RevisionId': str(response['Version']),
+                # 'Version': response['Version'],
                 }
 
 ###--------------------------------------------------------------------------------------------
@@ -196,7 +201,7 @@ class AwsSsmParameterProvider(ParameterProvider):
         if not found:
             raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] in ['String']:
+            if not found[0]['Type'] == 'String':
                 raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM parameter: path={found[0]['Name']}")
         response = ssm.get_parameter_history(Name=found[0]['Name'])
@@ -204,25 +209,27 @@ class AwsSsmParameterProvider(ParameterProvider):
         if revision is None:
             ### get the latest revision
             result = {
-                    'Revision': parameters[0]['Version'],
+                    'RevisionId': str(parameters[0]['Version']),
+                    'Key': key, 
                     'Value': parameters[0]['Value'],
+                    'Path': found[0]['Name'][:-len(key)-1],
                     'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'User': parameters[0]['LastModifiedUser'],
-                    'Key': key, 
-                    'Path': found[0]['Name'],
+                    # 'Version': parameters[0]['Version'],
                     }
         else:
             ### get specific revision
-            parameters = [x for x in parameters if str(x['Version']) == str(revision)]
+            parameters = [x for x in parameters if str(x['Version']) == revision]
             if not parameters:
                 raise DSOException(f"Revision '{revision}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
             result = {
-                    'Revision': parameters[0]['Version'],
+                    'RevisionId':str(parameters[0]['Version']),
+                    'Key': key, 
                     'Value': parameters[0]['Value'],
+                    'Path': found[0]['Name'][:-len(key)-1],
                     'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'User': parameters[0]['LastModifiedUser'],
-                    'Key': key, 
-                    'Path': found[0]['Name'],
+                    # 'Version': parameters[0]['Version'],
                     }
 
         return result
@@ -234,19 +241,21 @@ class AwsSsmParameterProvider(ParameterProvider):
         if not found:
             raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] in ['String']:
+            if not found[0]['Type'] == 'String':
                 raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM parameter: path={found[0]['Name']}")
         response = ssm.get_parameter_history(Name=found[0]['Name'])
         parameters = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         result = { "Revisions":
             [{
-                'Revision': parameter['Version'],
+                'RevisionId': str(parameter['Version']),
+                'Key': key,
                 'Value': parameter['Value'],
+                'Path': found[0]['Name'][:-len(key)-1],
                 'Date': parameter['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                 'User': parameter['LastModifiedUser'],
-                'Key': key,
-                'Path': found[0]['Name'],
+                # 'Version': parameter['Version'],
+
             } for parameter in parameters]
         }
 
@@ -257,16 +266,16 @@ class AwsSsmParameterProvider(ParameterProvider):
     def delete(self, project, application, stage, key):
         ### only parameters owned by the stage can be deleted, hence uninherited=True
         found = self.locate_parameter(project, application, stage, key, uninherited=True)
-        if not found or len(found) == 0:
+        if not found:
             raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
             # if len(found) > 1:
             #     Logger.warn(f"More than one parameter found at '{found[0]['Name']}'. The first one taken, and the rest were discarded.")
-            if not found[0]['Type'] in ['String']:
+            if not found[0]['Type'] == 'String':
                 raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Deleting SSM parameter: path={found[0]['Name']}")
         response = ssm.delete_parameter(Name=found[0]['Name'])
-        return {'Key': key, 'Path': found[0]['Name']}
+        return {'Key': key, 'Path': found[0]['Name'][:-len(key)-1]}
 
 ###--------------------------------------------------------------------------------------------
 ###--------------------------------------------------------------------------------------------
