@@ -5,7 +5,7 @@ import json
 import pathlib
 import numbers
 from dsocli.logger import Logger
-from dsocli.config import Config
+from dsocli.config import Configs
 from dsocli.parameters import ParameterProvider
 from dsocli.stages import Stages
 from dsocli.constants import *
@@ -20,7 +20,7 @@ defaults = {
 }
 
 config = {
-    'parameters_dir' : '{0}/parameters'.format(Config.dir_name),
+    'parameters_dir' : '{0}/parameters'.format(Configs.dir_name),
     'file_extension' : {
         'yaml' : 'yml',
         'json' : 'json',
@@ -36,7 +36,7 @@ class LocalParameterProvider(ParameterProvider):
 ###--------------------------------------------------------------------------------------------
 
     def __get_param_file_extension(self):
-        format = Config.parameter_spec('format')
+        format = Configs.parameter_spec('format')
         if format is None:
             Logger.debug(f"'format' is not set for the parameter provider, defaulted to '{defaults['format']}'.")
             format = defaults['format']
@@ -47,13 +47,13 @@ class LocalParameterProvider(ParameterProvider):
 
     def __get_params_root_path(self, context=None):
         if not context:
-            return f"{Config.working_dir}/{config['parameters_dir']}"
+            return f"{Configs.working_dir}/{config['parameters_dir']}"
         stage, env = Stages.parse_context(context)
         # if env == 'default':
         #     return f"{params_root_path}/{stage}"
         # else:
         #     return f"{params_root_path}/{stage}/{env}"
-        return f"{Config.working_dir}/{config['parameters_dir']}/{stage}/{env}"
+        return f"{Configs.working_dir}/{config['parameters_dir']}/{stage}/{env}"
 
 ###--------------------------------------------------------------------------------------------
 
@@ -188,7 +188,7 @@ class LocalParameterProvider(ParameterProvider):
 
     def get_key_validator(self):
 
-        allowGroups = Config.parameter_spec('allowGroups')
+        allowGroups = Configs.parameter_spec('allowGroups')
         if allowGroups is None:
             Logger.debug(f"'allowGroups' is not set for the parameter provider, defaulted to '{defaults['allowGroups']}'.")
             allowGroups = defaults['allowGroups']
@@ -271,17 +271,17 @@ class LocalParameterProvider(ParameterProvider):
                         else:
                             grouppedParameters[group] = params
 
-        prependGroups = Config.parameter_spec('prependGroups')
+        prependGroups = Configs.parameter_spec('prependGroups')
         if prependGroups is None:
             Logger.debug(f"'prependGroups' is not set for the parameter provider, defaulted to '{defaults['prependGroups']}'.")
             prependGroups = defaults['prependGroups']
 
-        groupDelimiter = Config.parameter_spec('groupDelimiter') 
+        groupDelimiter = Configs.parameter_spec('groupDelimiter') 
         if groupDelimiter is None:
             Logger.debug(f"'groupDelimiter' is not set for the parameter provider, defaulted to '{defaults['groupDelimiter']}'.")
             groupDelimiter = defaults['groupDelimiter']
 
-        nestedDelimiter = Config.parameter_spec('nestedDelimiter')
+        nestedDelimiter = Configs.parameter_spec('nestedDelimiter')
         if nestedDelimiter is None:
             Logger.debug(f"'nestedDelimiter' is not set for the parameter provider, defaulted to '{defaults['nestedDelimiter']}'.")
             nestedDelimiter = defaults['nestedDelimiter']
