@@ -38,19 +38,18 @@ class AwsSsmSecretProvider(SecretProvider):
 ###--------------------------------------------------------------------------------------------
 
     def get_secret_prefix(self, project, application, stage, key=None):
-        # output = f"/dso/{project}/{application}/{stage}"
-        output = "/dso"
-        output += f"/{project}"
+        result = "/dso/secrets"
+        result += f"/{project}"
         ### every application must belong to a project, no application overrides allowed in the default project
         if not project == 'default':
-            output += f"/{application}"
+            result += f"/{application}"
         else:
-            output += "/default"
+            result += "/default"
         stage = Stages.normalize(stage)
-        output += f"/{stage}"
+        result += f"/{stage}"
         if key:
-            output += f"/{key}"
-        return output
+            result += f"/{key}"
+        return result
 
 
 ###--------------------------------------------------------------------------------------------
@@ -74,9 +73,9 @@ class AwsSsmSecretProvider(SecretProvider):
         ### check children secrets
         path = self.get_secret_prefix(project, application, stage, key)
         # secrets = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['SecureString']},{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
-        secrets = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
-        if len(secrets['Parameters']) > 0:
-            raise DSOException("Secret key '{0}' is not allowed in the given stage becasue it would overwrite all the secrets in '{0}.*', such as '{0}.{1}'.".format(key,secrets['Parameters'][0]['Name'][len(path)+1:]))
+        response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
+        if len(response['Parameters']) > 0:
+            raise DSOException("Secret key '{0}' is not allowed in the given context becasue it would overwrite '{0}.{1}' and all other secrets in '{0}.*' scope if any.".format(key,response['Parameters'][0]['Name'][len(path)+1:]))
 
         ### check parent secrets
         scopes = key.split('.')
@@ -85,21 +84,22 @@ class AwsSsmSecretProvider(SecretProvider):
             path = self.get_secret_prefix(project, application, stage, subKey)
             Logger.debug(f"Describing secrets: path={path}")
             # secrets = ssm.describe_parameters(ParameterFilters=[{'Key':'Type', 'Values':['SecureString']},{'Key':'Name', 'Values':[path]}])
-            secrets = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
-            if len(secrets['Parameters']) > 0:
-                raise DSOException("Secret key '{0}' is not allowed in the given stage becasue it would overwrite secret '{1}'.".format(key, subKey))
+            response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
+            if len(response['Parameters']) > 0:
+                raise DSOException("Secret key '{0}' is not allowed in the given context becasue it would overwrite secret '{1}'.".format(key, subKey))
 
 ###--------------------------------------------------------------------------------------------
 
     def locate_secret(self, project, application, stage, key, uninherited=False):
         Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
         paths = self.get_hierachy_paths(project, application, stage, key, uninherited)
-        Logger.debug(f"SSM paths to search in order: {paths}")
+        # Logger.debug(f"SSM paths to search in order: {paths}")
         for path in paths:
             Logger.debug(f"Describing SSM secrets: path={path}")
             # result = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['SecureString']},{'Key':'Name', 'Values':[path]}])
-            result = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
-            if len(result['Parameters']) > 0: return result['Parameters']
+            response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
+            if len(response['Parameters']) > 0: return response['Parameters']
+        return []
 
 ###--------------------------------------------------------------------------------------------
 
@@ -113,8 +113,8 @@ class AwsSsmSecretProvider(SecretProvider):
             if key in secrets:
                 Logger.warn("Inherited secret '{0}' overridden.".format(key))
             secrets[key] = {'Value': secret['Value'], 
-                                'Path': path,
-                                'Revision': secret['Version'],
+                                'Path': secret['Name'][:-len(key)-1],
+                                'Version': secret['Version'],
                                 'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                             }
         return secrets
@@ -126,40 +126,42 @@ class AwsSsmSecretProvider(SecretProvider):
         if uninherited:
             paths.append(self.get_secret_prefix(project, application, stage, key))
         else:
-            ### check /dso/project/application/stage/env
-            paths.append(self.get_secret_prefix(project, application, stage, key))
-            if not Stages.is_stage_default_env(stage): ### otherwise already added above
-                ### check /dso/project/application/stage/default
-                paths.append(self.get_secret_prefix(project, application, Stages.get_stage_default_env(stage), key))
-            if not Stages.is_default(stage): ### otherwise already added above
-                ### check /dso/project/application/default
-                 paths.append(self.get_secret_prefix(project, application, Stages.default_stage(), key))
-            if not application == 'default': ### otherwise already added above
-                ### check /dso/project/default/stage/env
-                paths.append(self.get_secret_prefix(project, 'default', stage, key))
-                if not Stages.is_stage_default_env(stage): ### otherwise already added above
-                    ### check /dso/project/default/stage/default
-                    paths.append(self.get_secret_prefix(project, 'default', Stages.get_stage_default_env(stage), key))
-                if not Stages.is_default(stage): ### otherwise already added above
-                    ### check /dso/project/default/default
-                    paths.append(self.get_secret_prefix(project, 'default', Stages.default_stage(), key))
-                if not project == 'default': ### otherwise already added above
-                    ### check /dso/default/default/stage/env
+            ### Add the global context: /dso/default/default/default/0
+            paths.append(self.get_secret_prefix('default', 'default', 'default/0', key))
+            if not Stages.is_default(stage):
+                ### Add global stage context
+                paths.append(self.get_secret_prefix('default', 'default', Stages.get_stage_default_env(stage), key))
+                ### Add global numbered stage context
+                if not Stages.is_stage_default_env(stage):
                     paths.append(self.get_secret_prefix('default', 'default', stage, key))
-                    if not Stages.is_stage_default_env(stage): ### otherwise already added above
-                        ### check /dso/default/default/stage/default
-                        paths.append(self.get_secret_prefix('default', 'default', Stages.get_stage_default_env(stage), key))
-                    if not Stages.is_default(stage): ### otherwise already added above
-                        ### check /dso/default/default/default
-                        paths.append(self.get_secret_prefix('default', 'default', Stages.default_stage(), key))
+
+            if not project == 'default':
+                ### Add the project context: /dso/project/default/default/0
+                paths.append(self.get_secret_prefix(project, 'default', 'default/0', key))
+                if not Stages.is_default(stage):
+                    ### Add the project stage context: /dso/project/default/stage/0
+                    paths.append(self.get_secret_prefix(project, 'default', Stages.get_stage_default_env(stage), key))
+                    ### Add the project numbered stage context: /dso/project/default/stage/env
+                    if not Stages.is_stage_default_env(stage):
+                        paths.append(self.get_secret_prefix(project, 'default', stage, key))
+                
+                if not application == 'default':
+                    ### Add the application context: /dso/project/application/default/0
+                    paths.append(self.get_secret_prefix(project, application, 'default/0', key))
+                    if not Stages.is_default(stage):
+                        ### Add the project stage context: /dso/project/application/stage/0
+                        paths.append(self.get_secret_prefix(project, application, Stages.get_stage_default_env(stage), key))
+                        ### Add the application numbered stage context: /dso/project/application/stage/env
+                        if not Stages.is_stage_default_env(stage):
+                            paths.append(self.get_secret_prefix(project, application, stage, key))
 
         return paths
 
 ###--------------------------------------------------------------------------------------------
 
     def list(self, project, application, stage, uninherited, decrypt):
-        ### construct search path in hierachy with no key specified in reverse order
-        paths = list(reversed(self.get_hierachy_paths(project, application, stage, None, uninherited)))
+        ### construct search path in hierachy with no key specified
+        paths = list(self.get_hierachy_paths(project, application, stage, None, uninherited))
         # Logger.debug(f"SSM paths to search in order: {paths}")
         secrets = {}
         for path in paths:
@@ -170,8 +172,9 @@ class AwsSsmSecretProvider(SecretProvider):
             result['Secrets'].append({'Key': item[0], 
                                         'Value': item[1]['Value'], 
                                         'Path': item[1]['Path'],
-                                        'Revision': item[1]['Revision'],
+                                        'RevisionId': str(item[1]['Version']),
                                         'Date': item[1]['Date'],
+                                        # 'Version': item[1]['Version'],
                                         })
 
         return result
@@ -180,15 +183,17 @@ class AwsSsmSecretProvider(SecretProvider):
 
     def add(self, project, application, stage, key, value):
         self.assert_no_scope_overwrites(project, application, stage, key)
-        found = self.locate_secret(project, application, stage, key, True)
-        if found and len(found) > 0 and not found[0]['Type'] in ['SecureString']:
-            raise DSOException(f"Failed to add secret '{key}' becasue there is already a SSM parameter with the same key in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        found = self.locate_secret(project, application, stage, key, uninherited=True)
+        if found and not found[0]['Type'] == 'SecureString':
+            raise DSOException(f"Failed to add secret '{key}' becasue becasue the key is not available in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         path = self.get_secret_prefix(project, application, stage=stage, key=key)
         Logger.info(f"Adding SSM secret: path={path}")
         response = ssm.put_parameter(Name=path, Value=value, Type='SecureString', Overwrite=True)
         return {'Key': key, 
-                'Path': path,
-                'Revision': response['Version'],
+                'Value': value,
+                'Path': path[:-len(key)-1],
+                'RevisionId': str(response['Version']),
+                # 'Version': response['Version'],
                 }
 
 ###--------------------------------------------------------------------------------------------
@@ -198,7 +203,7 @@ class AwsSsmSecretProvider(SecretProvider):
         if not found:
             raise DSOException(f"Secret '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] in ['SecureString']:
+            if not found[0]['Type'] == 'SecureString':
                 raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM secret: path={found[0]['Name']}")
         response = ssm.get_parameter_history(Name=found[0]['Name'], WithDecryption=True)
@@ -206,25 +211,27 @@ class AwsSsmSecretProvider(SecretProvider):
         if revision is None:
             ### get the latest revision
             result = {
-                    'Revision': secrets[0]['Version'],
+                    'RevisionId': str(secrets[0]['Version']),
+                    'Key': key, 
                     'Value': secrets[0]['Value'],
+                    'Path': found[0]['Name'][:-len(key)-1],
                     'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'User': secrets[0]['LastModifiedUser'],
-                    'Key': key, 
-                    'Path': found[0]['Name'],
+                    # 'Version': secrets[0]['Version'],
                     }
         else:
             ### get specific revision
-            secrets = [x for x in secrets if str(x['Version']) == str(revision)]
+            secrets = [x for x in secrets if str(x['Version']) == revision]
             if not secrets:
                 raise DSOException(f"Revision '{revision}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
             result = {
-                    'Revision': secrets[0]['Version'],
+                    'RevisionId': str(secrets[0]['Version']),
+                    'Key': key, 
                     'Value': secrets[0]['Value'],
+                    'Path': found[0]['Name'][:-len(key)-1],
                     'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'User': secrets[0]['LastModifiedUser'],
-                    'Key': key, 
-                    'Path': found[0]['Name'],
+                    # 'Version': secrets[0]['Version'],
                     }
 
         return result
@@ -236,19 +243,20 @@ class AwsSsmSecretProvider(SecretProvider):
         if not found:
             raise DSOException(f"Secret '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] in ['SecureString']:
+            if not found[0]['Type'] == 'SecureString':
                 raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM secret: path={found[0]['Name']}")
         response = ssm.get_parameter_history(Name=found[0]['Name'], WithDecryption=True)
         secrets = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         result = { "Revisions":
             [{
-                'Revision': secret['Version'],
+                'RevisionId': str(secret['Version']),
+                'Key': key,
                 'Value': secret['Value'],
+                'Path': found[0]['Name'][:-len(key)-1],
                 'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                 'User': secret['LastModifiedUser'],
-                'Key': key,
-                'Path': found[0]['Name'],
+                # 'Version': secret['Version'],
             } for secret in secrets]
         }
 
@@ -259,16 +267,16 @@ class AwsSsmSecretProvider(SecretProvider):
     def delete(self, project, application, stage, key):
         ### only secrets owned by the stage can be deleted, hence uninherited=True
         found = self.locate_secret(project, application, stage, key, uninherited=True)
-        if not found or len(found) == 0:
+        if not found:
                 raise DSOException(f"Secret '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
             # if len(found) > 1:
             #     Logger.warn(f"More than one secret found at '{found[0]['Name']}'. The first one taken, and the rest were discarded.")
-            if not found[0]['Type'] in ['SecureString']:
+            if not found[0]['Type'] == 'SecureString':
                 raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Deleting SSM secret: path={found[0]['Name']}")
         ssm.delete_parameter(Name=found[0]['Name'])
-        return {'Key': key, 'Path': found[0]['Name']}
+        return {'Key': key, 'Path': found[0]['Name'][:-len(key)-1]}
 
 ###--------------------------------------------------------------------------------------------
 ###--------------------------------------------------------------------------------------------
