@@ -19,7 +19,7 @@ class StoreProvider(ProviderBase):
         Logger.debug(f"Validating: key={key}")
         pattern = self.get_key_validator()
         if not re.match(pattern, key):
-            raise DSOException(CLI_MESSAGES['InvalidKey'].format(key, pattern))
+            raise DSOException(MESSAGES['InvalidKey'].format(key, pattern))
 
     def get_key_validator(self, key):
         raise NotImplementedError()
