@@ -16,7 +16,7 @@ from .stages import Stages
 class TemplateProvider(StoreProvider):
     def list(self):
         raise NotImplementedError()
-    def add(self, key, content, render_path):
+    def add(self, key, contents, render_path):
         raise NotImplementedError()
     def delete(self, key, starts_with):
         raise NotImplementedError()
@@ -29,7 +29,12 @@ class TemplateManager():
     def default_render_path(self):
         return Configs.working_dir
 
-    def list(self, uninherited):
+    def validate_key(self, key):
+        provider = Providers.TemplateProvider()
+        Logger.info("Start validating template key...")
+        return provider.validate_key(key)
+
+    def list(self, uninherited=False):
         project = Configs.project
         application = Configs.application
         provider = Providers.TemplateProvider()
@@ -50,7 +55,7 @@ class TemplateManager():
         
         return response
 
-    def add(self, key, content, render_path):
+    def add(self, key, contents, render_path):
         self.validate_key(key)
         project = Configs.project
         application = Configs.application
@@ -71,11 +76,12 @@ class TemplateManager():
         if os.path.isabs(render_path):
             Logger.warn(f"Render path {render_path} is not releative to the application root.")
         else:
-            if not render_path.startswith(f".{os.sep}"):
+            if not (render_path == '.' or render_path.startswith(f".{os.sep}")):
                 render_path = os.path.join(f".{os.sep}", render_path)
+
         try:
             Logger.info(f"Start adding template: project={project}, application={application}, key={key}, render_path={render_path}")
-            result = provider.add(project, application, key, content)
+            result = provider.add(project, application, key, contents)
             result['RenderTo'] = render_path
         finally:
             if os.path.abspath(render_path) == os.path.abspath(os.path.join(self.default_render_path, key)):
@@ -84,13 +90,21 @@ class TemplateManager():
                 Configs.register_template_custom_render_path(key, render_path)
         return result
 
-    def get(self, key):
+    def get(self, key, revision=None):
         # self.validate_key(key)
         project = Configs.project
         application = Configs.application
         provider = Providers.TemplateProvider()
-        Logger.info(f"Start getting template: project={project}, application={application}, key={key}")
-        return provider.get(project, application, key)
+        Logger.info(f"Start getting template contents: project={project}, application={application}, key={key}")
+        return provider.get(project, application, key, revision)
+
+    def history(self, key):
+        # self.validate_key(key)
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.TemplateProvider()
+        Logger.info(f"Start getting template history: project={project}, application={application}, key={key}")
+        return provider.history(project, application, key)
 
     def delete(self, key):
         # self.validate_key(key)
@@ -104,11 +118,6 @@ class TemplateManager():
             Configs.unregister_template_custom_render_path(key)
         return result
 
-    def validate_key(self, key):
-        provider = Providers.TemplateProvider()
-        Logger.info("Start validating template key...")
-        return provider.validate_key(key)
-
     def render(self, stage, limit=''):
         project = Configs.project
         application = Configs.application
@@ -129,9 +138,7 @@ class TemplateManager():
         merge_dicts(deflatten_dict({x['Key']: x['Value'] for x in secrets['Secrets']}), merged)
 
         Logger.info("Loading templates...")
-        provider = Providers.TemplateProvider()
-        templates = provider.list(project, application, uninherited=False)['Templates']
-        renderPaths = Configs.get_template_render_paths()
+        templates = self.list()['Templates']
 
         jinja_env = jinja2.Environment(undefined=jinja2.StrictUndefined)
 
@@ -139,13 +146,17 @@ class TemplateManager():
         rendered = []
         for item in templates:
             key = item['Key']
-
             if not key.startswith(limit): continue
 
-            template = provider.get(project, application, key)
+            renderPath = item['RenderTo']
+            if os.path.isdir(renderPath):
+                raise DSOException("There is an existing directory at the template render path '{renderPath}'.")
+            if os.path.dirname(renderPath):
+                os.makedirs(os.path.dirname(renderPath), exist_ok=True)
+
             Logger.debug(f"Rendering template: key={key}")
             try:
-                template = jinja_env.from_string(template)
+                template = jinja_env.from_string(self.get(key)['Contents'])
             except:
                 Logger.error(f"Failed to load template: {key}")
                 raise
@@ -154,22 +165,11 @@ class TemplateManager():
             #     Logger.warn(f"Undecalared parameter(s) found:\n{set(undeclaredParams)}")
             try:
                 renderedContent = template.render(merged)
-            except:
+            except Exception as e:
                 Logger.error(f"Failed to render template: {key}")
-                raise
+                msg = getattr(e, 'message', getattr(e, 'msg', str(e)))
+                raise DSOException(msg)
 
-            if key in renderPaths:
-                renderPath = renderPaths[key]
-            else:
-                renderPath = os.path.join(self.default_render_path, key)
-
-            if not (renderPath == '.' or renderPath.startswith(f'.{os.sep}')):
-                renderPath = os.path.join('./', renderPath)
-
-            # outputFilePath = os.path.join(renderPath, key)
-            # outputFilePath = os.path.join(renderPath, key)
-            if os.path.dirname(renderPath):
-                os.makedirs(os.path.dirname(renderPath), exist_ok=True)
             with open(renderPath, 'w', encoding='utf-8') as f:
                 f.write(renderedContent)
             
